package httpServer;
import java.io.DataInputStream;
import java.io.DataOutputStream;

import javax.microedition.io.*;

public class ServerThread extends Thread {

	ServerSocketConnection ssConn;
	Tc65SerialPort	tc65SerialPort;

	// Network timeout. Can be very big, looks like it does not affect socket release time. 
	private final static int NetworkTimeout = 864000;
	
	public volatile boolean bStop = false; // Set it to true to stop thread
	public int Port = 80;
	
//	 set the following to true to invoke WorkerThread and handle multiple connections
	public boolean bRunMultithreadedServer = true; 

	public boolean bShowDebug = false;
	
	private void DbgPrint(String s)
	{
		if(bShowDebug == true)
			tc65SerialPort.serialOut(s);
	}
	private void DbgPrint(char c)
	{
		if(bShowDebug == true)
			tc65SerialPort.serialOut(c);
	}
	
	public ServerThread(Tc65SerialPort tc65SerialPort) {
//		super();
		this.tc65SerialPort = tc65SerialPort;
		
		DbgPrint("Server::Server()\r\n");
	}

	public void run()
	{
//		 Create the server listening socket for port 1234 
//		   ServerSocketConnection scn = (ServerSocketConnection)
//		                            Connector.open("socket://:1234");

		   // Wait for a connection.
//		   SocketConnection sc = (SocketConnection) scn.acceptAndOpen();

		DbgPrint("Server.run() using " + ((bRunMultithreadedServer == false)?"single ":"multiple ") + "thread(s)\r\n");
		
//		 create the server                      
		while (bStop == false) 
		{
			try {
				ssConn = (ServerSocketConnection) Connector
						.open("socket://:" + Port + ";timeout=" + NetworkTimeout); // avoid "Network idle timeout" exception
		 
				 while (bStop == false) 
			    {                       
					try {
						// wait for incoming connection   
						DbgPrint("Server listens at " + ssConn.getLocalAddress() +
								":" + ssConn.getLocalPort() + " (" + Thread.activeCount() + " threads active)\r\n");
						SocketConnection sc = (SocketConnection) ssConn.acceptAndOpen();
						DbgPrint("SocketConnection: acceptAndOpen() worked out\r\n");
						if(bRunMultithreadedServer == true)
						{
							// hand new stream to worker thread which than handles the data I/O                
							WorkerThread myWorker = new WorkerThread(sc, tc65SerialPort);
							myWorker.start();
						}
						else
							ProcessSocketConnection(sc);
					} catch (Exception e) {
						DbgPrint("SocketConnection.acceptAndOpen(): exception " + 
								e.getMessage() + "\r\n");
						DbgPrint("Closing ssConn\r\n"); 
						ssConn.close();
						if(bStop == false)
						{
							DbgPrint("Waiting 60 sec to release socket\r\n");
							// Waiting to completely release socket 
							Thread.sleep(60000);
						}
						if(bStop == false)
						{
							DbgPrint("Reopening ServerSocketConnection\r\n"); 
						ssConn = (ServerSocketConnection) Connector
							.open("socket://:" + Port + ";timeout=" + NetworkTimeout); 
						// avoid "Network idle timeout" exception
						}
					}	 
			    }
			} catch (Exception e) {
				DbgPrint("Server.Connector.open(): exception " + e.getMessage() + "\r\n");
			}		 
			if(bStop == false)
			{
				try {Thread.sleep(10000);} catch (Exception e) {}
			}
		} //while (bStop == false)

		try {ssConn.close();} catch (Exception e) {}
		
		try {
			DbgPrint("ServerThread: calling interrupt()\r\n");
			interrupt();
			DbgPrint("ServerThread: called interrupt()\r\n");
			DbgPrint("ServerThread: calling join()\r\n");
			join();
			DbgPrint("ServerThread: called join()\r\n");
		} catch (Exception e) {
			DbgPrint("Server.run(): cannot join(), exception " + 
					e.getMessage() + "\r\n");
		}
		DbgPrint("Server.run(): returning\r\n");
	}
	
	// Handle socket conn without using threads
	private void ProcessSocketConnection(SocketConnection sc)
	{
		try {
			DbgPrint("Connection from: " + sc.getAddress()
					+ ":" + sc.getPort() + "\r\n");
		} catch (Exception e) {
			DbgPrint("ProcessSocketConnection: failed to get remote addr: excp: " + e.getMessage() + "\r\n");
		}		
		try {
			// Set application specific hints on the socket.
			sc.setSocketOption(SocketConnection.DELAY, 0);
			sc.setSocketOption(SocketConnection.LINGER, 0);
			sc.setSocketOption(SocketConnection.KEEPALIVE, 0);
			sc.setSocketOption(SocketConnection.RCVBUF, 128);
			sc.setSocketOption(SocketConnection.SNDBUF, 128);
		} catch (Exception e) {
			DbgPrint("ProcessSocketConnection: sc.setSocketOption() excp: " + e.getMessage() + "\r\n");
		}	   
		try {
			// Get the input stream of the connection.
			DbgPrint("ProcessSocketConnection: opening DataInputStream\r\n");
			DataInputStream is = sc.openDataInputStream();
			// Get the output stream of the connection.
			DbgPrint("ProcessSocketConnection: opening DataOutputStream\r\n");
			DataOutputStream os = sc.openDataOutputStream();
			// Read the input data until 1s timeout happens.
			String result = new String("<html><body><b>Modem received:</b><br>");//is.readUTF();
			int timeout = (int)System.currentTimeMillis();
			while((int)System.currentTimeMillis() - timeout < 3000)
			{
				if(is.available() > 0)
				{
					char b = (char)is.read();
					// Convert special chars into HTML
					switch(b)
					{
					case 0x0d:
						result += "<br>";
						break;
					case 0x0a:
						break;
					default:
						result += b;
						break;
					}
					DbgPrint(b);
					timeout = (int)System.currentTimeMillis();
				}
			}
			DbgPrint("\r\n\r\nTimeout in incoming data\r\n" + 
					"ProcessSocketConnection: read " + result.length() + "bytes: " + "\r\n");
					
			// Echo the data back to the sender.
			result += "</body></html>";
			byte[] arOut = result.getBytes();
			os.write( arOut );

			DbgPrint("ProcessSocketConnection: sent response, closing all\r\n");
			// Close everything.
			is.close();
			os.close();
			sc.close();
		} catch (Exception e) {
			DbgPrint("ProcessSocketConnection: comms: excp: " + e.getMessage() + "\r\n");
		}	
		
	}
	public void stopThread()
	{
		DbgPrint("Called Server.stopThread()\r\n");
		bStop = true;
		try {
			ssConn.close();
		} catch (Exception e) {
			DbgPrint("Server.stop(): exception " + e.getMessage() + "\r\n");
		}	
	}
}

